@@ -1,6 +1,5 @@
 use crate::build_context::BridgeModel;
-use crate::python_interpreter::InterpreterKind;
-use crate::{BuildContext, PlatformTag, PythonInterpreter};
+use crate::{BuildContext, PlatformTag, PythonInterpreter, Target};
 use anyhow::{anyhow, bail, Context, Result};
 use fat_macho::FatWriter;
 use fs_err::{self as fs, File};
@@ -22,11 +21,20 @@ pub fn compile(
     context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
     bindings_crate: &BridgeModel,
-) -> Result<HashMap<String, PathBuf>> {
+) -> Result<Vec<HashMap<String, PathBuf>>> {
+    let root_pkg = context.cargo_metadata.root_package().unwrap();
+    let targets: Vec<_> = root_pkg
+        .targets
+        .iter()
+        .filter(|target| match bindings_crate {
+            BridgeModel::Bin(_) => target.kind.contains(&"bin".to_string()),
+            _ => target.kind.contains(&"cdylib".to_string()),
+        })
+        .collect();
     if context.target.is_macos() && context.universal2 {
-        compile_universal2(context, python_interpreter, bindings_crate)
+        compile_universal2(context, python_interpreter, bindings_crate, &targets)
     } else {
-        compile_target(context, python_interpreter, bindings_crate)
+        compile_targets(context, python_interpreter, bindings_crate, &targets)
     }
 }
 
@@ -35,23 +43,33 @@ fn compile_universal2(
     context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
     bindings_crate: &BridgeModel,
-) -> Result<HashMap<String, PathBuf>> {
+    targets: &[&cargo_metadata::Target],
+) -> Result<Vec<HashMap<String, PathBuf>>> {
     let build_type = if bindings_crate.is_bin() {
         "bin"
     } else {
         "cdylib"
     };
     let mut aarch64_context = context.clone();
-    aarch64_context.cargo_extra_args.extend(vec![
-        "--target".to_string(),
-        "aarch64-apple-darwin".to_string(),
-    ]);
+    aarch64_context.target = Target::from_target_triple(Some("aarch64-apple-darwin".to_string()))?;
 
-    let aarch64_artifact = compile_target(&aarch64_context, python_interpreter, bindings_crate)
-        .context("Failed to build a aarch64 library through cargo")?
-        .get(build_type)
-        .cloned()
-        .ok_or_else(|| {
+    let aarch64_artifacts = compile_targets(
+        &aarch64_context,
+        python_interpreter,
+        bindings_crate,
+        targets,
+    )
+    .context("Failed to build a aarch64 library through cargo")?;
+    let mut x86_64_context = context.clone();
+    x86_64_context.target = Target::from_target_triple(Some("x86_64-apple-darwin".to_string()))?;
+
+    let x86_64_artifacts =
+        compile_targets(&x86_64_context, python_interpreter, bindings_crate, targets)
+            .context("Failed to build a x86_64 library through cargo")?;
+
+    let mut universal_artifacts = Vec::with_capacity(targets.len());
+    for (aarch64_artifact, x86_64_artifact) in aarch64_artifacts.iter().zip(x86_64_artifacts) {
+        let aarch64_artifact = aarch64_artifact.get(build_type).cloned().ok_or_else(|| {
             if build_type == "cdylib" {
                 anyhow!(
                     "Cargo didn't build an aarch64 cdylib. Did you miss crate-type = [\"cdylib\"] \
@@ -61,17 +79,7 @@ fn compile_universal2(
                 anyhow!("Cargo didn't build an aarch64 bin.")
             }
         })?;
-    let mut x86_64_context = context.clone();
-    x86_64_context.cargo_extra_args.extend(vec![
-        "--target".to_string(),
-        "x86_64-apple-darwin".to_string(),
-    ]);
-
-    let x86_64_artifact = compile_target(&x86_64_context, python_interpreter, bindings_crate)
-        .context("Failed to build a x86_64 library through cargo")?
-        .get(build_type)
-        .cloned()
-        .ok_or_else(|| {
+        let x86_64_artifact = x86_64_artifact.get(build_type).cloned().ok_or_else(|| {
             if build_type == "cdylib" {
                 anyhow!(
                     "Cargo didn't build a x86_64 cdylib. Did you miss crate-type = [\"cdylib\"] \
@@ -81,66 +89,74 @@ fn compile_universal2(
                 anyhow!("Cargo didn't build a x86_64 bin.")
             }
         })?;
+        // Create an universal dylib
+        let output_path = aarch64_artifact
+            .display()
+            .to_string()
+            .replace("aarch64-apple-darwin/", "");
+        let mut writer = FatWriter::new();
+        let aarch64_file = fs::read(aarch64_artifact)?;
+        let x86_64_file = fs::read(x86_64_artifact)?;
+        writer
+            .add(aarch64_file)
+            .map_err(|e| anyhow!("Failed to add aarch64 cdylib: {:?}", e))?;
+        writer
+            .add(x86_64_file)
+            .map_err(|e| anyhow!("Failed to add x86_64 cdylib: {:?}", e))?;
+        writer
+            .write_to_file(&output_path)
+            .map_err(|e| anyhow!("Failed to create universal cdylib: {:?}", e))?;
 
-    // Create an universal dylib
-    let output_path = aarch64_artifact
-        .display()
-        .to_string()
-        .replace("aarch64-apple-darwin/", "");
-    let mut writer = FatWriter::new();
-    let aarch64_file = fs::read(aarch64_artifact)?;
-    let x86_64_file = fs::read(x86_64_artifact)?;
-    writer
-        .add(aarch64_file)
-        .map_err(|e| anyhow!("Failed to add aarch64 cdylib: {:?}", e))?;
-    writer
-        .add(x86_64_file)
-        .map_err(|e| anyhow!("Failed to add x86_64 cdylib: {:?}", e))?;
-    writer
-        .write_to_file(&output_path)
-        .map_err(|e| anyhow!("Failed to create universal cdylib: {:?}", e))?;
+        let mut result = HashMap::new();
+        result.insert(build_type.to_string(), PathBuf::from(output_path));
+        universal_artifacts.push(result);
+    }
+    Ok(universal_artifacts)
+}
 
-    let mut result = HashMap::new();
-    result.insert(build_type.to_string(), PathBuf::from(output_path));
-    Ok(result)
+fn compile_targets(
+    context: &BuildContext,
+    python_interpreter: Option<&PythonInterpreter>,
+    bindings_crate: &BridgeModel,
+    targets: &[&cargo_metadata::Target],
+) -> Result<Vec<HashMap<String, PathBuf>>> {
+    let mut artifacts = Vec::with_capacity(targets.len());
+    for target in targets {
+        artifacts.push(compile_target(
+            context,
+            python_interpreter,
+            bindings_crate,
+            target,
+        )?);
+    }
+    Ok(artifacts)
 }
 
 fn compile_target(
     context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
     bindings_crate: &BridgeModel,
+    binding_target: &cargo_metadata::Target,
 ) -> Result<HashMap<String, PathBuf>> {
     let target = &context.target;
 
-    let mut shared_args: Vec<_> = context
-        .cargo_extra_args
-        .iter()
-        .map(String::as_str)
-        .collect();
+    let mut cargo_rustc: cargo_options::Rustc = context.cargo_options.clone().into();
+    cargo_rustc.message_format = vec!["json".to_string()];
 
-    if context.release {
-        let has_cargo_profile = shared_args
-            .iter()
-            .any(|arg| *arg == "--profile" || arg.starts_with("--profile="));
-        // --release and --profile are conflicting options
-        if !has_cargo_profile {
-            shared_args.push("--release");
-        }
+    // --release and --profile are conflicting options
+    if context.release && cargo_rustc.profile.is_none() {
+        cargo_rustc.release = true;
     }
-    let mut rustc_args: Vec<&str> = context
-        .rustc_extra_args
-        .iter()
-        .map(String::as_str)
-        .collect();
 
     let mut rust_flags = env::var_os("RUSTFLAGS");
 
-    // We need to pass --bins / --lib to set the rustc extra args later
-    // TODO: What do we do when there are multiple bin targets?
+    // We need to pass --bin / --lib
     match bindings_crate {
-        BridgeModel::Bin(..) => shared_args.push("--bins"),
+        BridgeModel::Bin(..) => {
+            cargo_rustc.bin.push(binding_target.name.clone());
+        }
         BridgeModel::Cffi | BridgeModel::Bindings(..) | BridgeModel::BindingsAbi3(..) => {
-            shared_args.push("--lib");
+            cargo_rustc.lib = true;
             // https://github.com/rust-lang/rust/issues/59302#issue-422994250
             // We must only do this for libraries as it breaks binaries
             // For some reason this value is ignored when passed as rustc argument
@@ -160,47 +176,51 @@ fn compile_target(
             format!("{base}.abi3.so", base = module_name)
         }
     };
-    // Change LC_ID_DYLIB to the final .so name for macOS targets to avoid linking with
-    // non-existent library.
-    // See https://github.com/PyO3/setuptools-rust/issues/106 for detail
-    let macos_dylib_install_name = format!("link-args=-Wl,-install_name,@rpath/{}", so_filename);
 
     // https://github.com/PyO3/pyo3/issues/88#issuecomment-337744403
     if target.is_macos() {
         if let BridgeModel::Bindings(..) | BridgeModel::BindingsAbi3(..) = bindings_crate {
-            let mac_args = &[
-                "-C",
-                "link-arg=-undefined",
-                "-C",
-                "link-arg=dynamic_lookup",
-                "-C",
-                &macos_dylib_install_name,
+            // Change LC_ID_DYLIB to the final .so name for macOS targets to avoid linking with
+            // non-existent library.
+            // See https://github.com/PyO3/setuptools-rust/issues/106 for detail
+            let macos_dylib_install_name =
+                format!("link-args=-Wl,-install_name,@rpath/{}", so_filename);
+            let mac_args = [
+                "-C".to_string(),
+                "link-arg=-undefined".to_string(),
+                "-C".to_string(),
+                "link-arg=dynamic_lookup".to_string(),
+                "-C".to_string(),
+                macos_dylib_install_name,
             ];
-            rustc_args.extend(mac_args);
+            cargo_rustc.args.extend(mac_args);
         }
+    } else if target.is_emscripten() {
+        let flags = rust_flags.get_or_insert_with(Default::default);
+        flags.push(" -Z link-native-libraries=no");
+        let emscripten_args = [
+            "-C".to_string(),
+            "link-arg=-sSIDE_MODULE=2".to_string(),
+            "-C".to_string(),
+            "link-arg=-sWASM_BIGINT".to_string(),
+        ];
+        cargo_rustc.args.extend(emscripten_args);
     }
 
     if context.strip {
-        rustc_args.extend(&["-C", "link-arg=-s"]);
+        cargo_rustc
+            .args
+            .extend(["-C".to_string(), "link-arg=-s".to_string()]);
     }
 
-    let cargo_args = vec!["--message-format", "json"];
-
-    let build_args: Vec<_> = cargo_args
-        .iter()
-        .chain(&shared_args)
-        .chain(&["--"])
-        .chain(&rustc_args)
-        .collect();
-
     let target_triple = target.target_triple();
     let mut build_command = if target.is_msvc() && target.cross_compiling() {
-        let mut build = cargo_xwin::Build::new(Some(context.manifest_path.clone()));
+        let mut build = cargo_xwin::Rustc::from(cargo_rustc);
 
         build.target = vec![target_triple.to_string()];
-        build.build_command("rustc")?
+        build.build_command()?
     } else {
-        let mut build = cargo_zigbuild::Build::new(Some(context.manifest_path.clone()));
+        let mut build = cargo_zigbuild::Rustc::from(cargo_rustc);
         if !context.zig {
             build.disable_zig_linker = true;
             if target.user_specified {
@@ -219,7 +239,7 @@ fn compile_target(
             };
             build.target = vec![zig_triple];
         }
-        build.build_command("rustc")?
+        build.build_command()?
     };
 
     if context.zig {
@@ -235,7 +255,6 @@ fn compile_target(
     }
 
     build_command
-        .args(&build_args)
         // We need to capture the json messages
         .stdout(Stdio::piped())
         // We can't get colored human and json messages from rustc as they are mutually exclusive,
@@ -248,7 +267,7 @@ fn compile_target(
 
     if let BridgeModel::BindingsAbi3(_, _) = bindings_crate {
         let is_pypy = python_interpreter
-            .map(|p| p.interpreter_kind == InterpreterKind::PyPy)
+            .map(|p| p.interpreter_kind.is_pypy())
             .unwrap_or(false);
         if !is_pypy && !target.is_windows() {
             let pyo3_ver = pyo3_version(&context.cargo_metadata)
@@ -268,14 +287,17 @@ fn compile_target(
             if bindings_crate.is_bindings("pyo3")
                 || bindings_crate.is_bindings("pyo3-ffi")
                 || (matches!(bindings_crate, BridgeModel::BindingsAbi3(_, _))
-                    && matches!(interpreter.interpreter_kind, InterpreterKind::PyPy))
+                    && interpreter.interpreter_kind.is_pypy())
             {
                 build_command.env("PYO3_PYTHON", &interpreter.executable);
             }
 
             // rust-cpython, and legacy pyo3 versions
             build_command.env("PYTHON_SYS_EXECUTABLE", &interpreter.executable);
-        } else if (bindings_crate.is_bindings("pyo3") || bindings_crate.is_bindings("pyo3-ffi"))
+        } else if (bindings_crate.is_bindings("pyo3")
+            || bindings_crate.is_bindings("pyo3-ffi")
+            || (matches!(bindings_crate, BridgeModel::BindingsAbi3(_, _))
+                && interpreter.interpreter_kind.is_pypy()))
             && env::var_os("PYO3_CONFIG_FILE").is_none()
         {
             let pyo3_config = interpreter.pyo3_config_file();
@@ -322,11 +344,18 @@ fn compile_target(
                 let crate_name = match package_in_metadata {
                     Some(package) => &package.name,
                     None => {
-                        // This is a spurious error I don't really understand
-                        println!(
-                            "⚠️  Warning: The package {} wasn't listed in `cargo metadata`",
-                            artifact.package_id
-                        );
+                        let package_id = &artifact.package_id;
+                        // Ignore the package if it's coming from Rust sysroot when compiling with `-Zbuild-std`
+                        let should_warn = !package_id.repr.contains("rustup")
+                            && !package_id.repr.contains("rustlib")
+                            && !artifact.features.contains(&"rustc-dep-of-std".to_string());
+                        if should_warn {
+                            // This is a spurious error I don't really understand
+                            println!(
+                                "⚠️  Warning: The package {} wasn't listed in `cargo metadata`",
+                                package_id
+                            );
+                        }
                         continue;
                     }
                 };
@@ -355,16 +384,10 @@ fn compile_target(
         .expect("Failed to wait on cargo child process");
 
     if !status.success() {
-        let command_str = build_args
-            .iter()
-            .map(ToString::to_string)
-            .collect::<Vec<String>>()
-            .join(" ");
         bail!(
-            r#"Cargo build finished with "{}": `cargo rustc --manifest-path {} {}`"#,
+            r#"Cargo build finished with "{}": `{:?}`"#,
             status,
-            context.manifest_path.display(),
-            command_str
+            build_command,
         )
     }
 
