@@ -1,3 +1,5 @@
+use crate::build_options::CargoOptions;
+use crate::target::Arch;
 use crate::BuildOptions;
 use crate::PlatformTag;
 use crate::PythonInterpreter;
@@ -14,31 +16,51 @@ use tempfile::TempDir;
 #[allow(clippy::too_many_arguments)]
 pub fn develop(
     bindings: Option<String>,
-    manifest_file: &Path,
-    cargo_extra_args: Vec<String>,
-    rustc_extra_args: Vec<String>,
+    cargo_options: CargoOptions,
     venv_dir: &Path,
     release: bool,
     strip: bool,
     extras: Vec<String>,
 ) -> Result<()> {
     let target = Target::from_target_triple(None)?;
+    let mut target_triple = None;
     let python = target.get_venv_python(&venv_dir);
+
+    // check python platform and architecture
+    match Command::new(&python)
+        .arg("-c")
+        .arg("import sysconfig; print(sysconfig.get_platform(), end='')")
+        .output()
+    {
+        Ok(output) if output.status.success() => {
+            let platform = String::from_utf8_lossy(&output.stdout);
+            if platform.contains("macos") {
+                if platform.contains("x86_64") && target.target_arch() != Arch::X86_64 {
+                    target_triple = Some("x86_64-apple-darwin".to_string());
+                } else if platform.contains("arm64") && target.target_arch() != Arch::Aarch64 {
+                    target_triple = Some("aarch64-apple-darwin".to_string());
+                }
+            }
+        }
+        _ => eprintln!("⚠️  Warning: Failed to determine python platform"),
+    }
+
     // Store wheel in a unique location so we don't get name clashes with parallel runs
     let wheel_dir = TempDir::new().context("Failed to create temporary directory")?;
 
     let build_options = BuildOptions {
         platform_tag: vec![PlatformTag::Linux],
         interpreter: vec![python.clone()],
+        find_interpreter: false,
         bindings,
-        manifest_path: Some(manifest_file.to_path_buf()),
         out: Some(wheel_dir.path().to_path_buf()),
         skip_auditwheel: false,
         zig: false,
-        target: None,
-        cargo_extra_args,
-        rustc_extra_args,
         universal2: false,
+        cargo: CargoOptions {
+            target: target_triple,
+            ..cargo_options
+        },
     };
 
     let build_context = build_options.into_build_context(release, strip, true)?;
