@@ -8,7 +8,7 @@ use cargo_zigbuild::Zig;
 use clap::{ArgEnum, IntoApp, Parser, Subcommand};
 use clap_complete::Generator;
 use maturin::{
-    develop, init_project, new_project, write_dist_info, BridgeModel, BuildOptions,
+    develop, init_project, new_project, write_dist_info, BridgeModel, BuildOptions, CargoOptions,
     GenerateProjectOptions, PathWriter, PlatformTag, PythonInterpreter, Target,
 };
 #[cfg(feature = "upload")]
@@ -19,32 +19,34 @@ use std::path::PathBuf;
 use std::str::FromStr;
 
 #[derive(Debug, Parser)]
-#[clap(name = env!("CARGO_PKG_NAME"), version)]
+#[clap(
+    version,
+    name = env!("CARGO_PKG_NAME"),
+    global_setting(clap::AppSettings::DeriveDisplayOrder)
+)]
 #[cfg_attr(feature = "cargo-clippy", allow(clippy::large_enum_variant))]
 /// Build and publish crates with pyo3, rust-cpython and cffi bindings as well
 /// as rust binaries as python packages
 enum Opt {
-    #[clap(name = "build")]
+    #[clap(name = "build", alias = "b")]
     /// Build the crate into python packages
     Build {
-        #[clap(flatten)]
-        build: BuildOptions,
-        /// Pass --release to cargo
+        /// Build artifacts in release mode, with optimizations
         #[clap(short = 'r', long)]
         release: bool,
         /// Strip the library for minimum file size
         #[clap(long)]
         strip: bool,
-        /// Don't build a source distribution
-        #[clap(long = "no-sdist")]
-        no_sdist: bool,
+        /// Build a source distribution
+        #[clap(long)]
+        sdist: bool,
+        #[clap(flatten)]
+        build: BuildOptions,
     },
     #[cfg(feature = "upload")]
     #[clap(name = "publish")]
     /// Build and publish the crate as python packages to pypi
     Publish {
-        #[clap(flatten)]
-        build: BuildOptions,
         /// Do not pass --release to cargo
         #[clap(long)]
         debug: bool,
@@ -56,11 +58,16 @@ enum Opt {
         no_sdist: bool,
         #[clap(flatten)]
         publish: PublishOpt,
+        #[clap(flatten)]
+        build: BuildOptions,
     },
     #[clap(name = "list-python")]
     /// Searches and lists the available python installations
-    ListPython,
-    #[clap(name = "develop")]
+    ListPython {
+        #[clap(long)]
+        target: Option<String>,
+    },
+    #[clap(name = "develop", alias = "dev")]
     /// Installs the crate as module in the current virtualenv
     ///
     /// Note that this command doesn't create entrypoints
@@ -68,30 +75,12 @@ enum Opt {
         /// Which kind of bindings to use. Possible values are pyo3, rust-cpython, cffi and bin
         #[clap(short = 'b', long = "bindings", alias = "binding-crate")]
         bindings: Option<String>,
-        #[clap(
-            short = 'm',
-            long = "manifest-path",
-            parse(from_os_str),
-            default_value = "Cargo.toml"
-        )]
-        /// The path to the Cargo.toml
-        manifest_path: PathBuf,
         /// Pass --release to cargo
         #[clap(short = 'r', long)]
         release: bool,
         /// Strip the library for minimum file size
         #[clap(long)]
         strip: bool,
-        /// Extra arguments that will be passed to cargo as `cargo rustc [...] [arg1] [arg2] --`
-        ///
-        /// Use as `--cargo-extra-args="--my-arg"`
-        #[clap(long = "cargo-extra-args")]
-        cargo_extra_args: Vec<String>,
-        /// Extra arguments that will be passed to rustc as `cargo rustc [...] -- [arg1] [arg2]`
-        ///
-        /// Use as `--rustc-extra-args="--my-arg"`
-        #[clap(long = "rustc-extra-args")]
-        rustc_extra_args: Vec<String>,
         /// Install extra requires aka. optional dependencies
         ///
         /// Use as `--extras=extra1,extra2`
@@ -103,6 +92,8 @@ enum Opt {
             multiple_occurrences = false
         )]
         extras: Vec<String>,
+        #[clap(flatten)]
+        cargo_options: CargoOptions,
     },
     /// Build only a source distribution (sdist) without compiling.
     ///
@@ -301,8 +292,11 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             manifest_path,
         } => {
             let build_options = BuildOptions {
-                manifest_path: Some(manifest_path),
                 out: Some(sdist_directory),
+                cargo: CargoOptions {
+                    manifest_path: Some(manifest_path),
+                    ..Default::default()
+                },
                 ..Default::default()
             };
             let build_context = build_options.into_build_context(false, false, false)?;
@@ -327,10 +321,10 @@ fn run() -> Result<()> {
             build,
             release,
             strip,
-            no_sdist,
+            sdist,
         } => {
             let build_context = build.into_build_context(release, strip, false)?;
-            if !no_sdist {
+            if sdist {
                 build_context.build_source_distribution()?;
             }
             let wheels = build_context.build_wheels()?;
@@ -361,10 +355,15 @@ fn run() -> Result<()> {
 
             upload_ui(&items, &publish)?
         }
-        Opt::ListPython => {
-            let target = Target::from_target_triple(None)?;
-            // We don't know the targeted bindings yet, so we use the most lenient
-            let found = PythonInterpreter::find_all(&target, &BridgeModel::Cffi, None)?;
+        Opt::ListPython { target } => {
+            let found = if target.is_some() {
+                let target = Target::from_target_triple(target)?;
+                PythonInterpreter::find_by_target(&target, None)
+            } else {
+                let target = Target::from_target_triple(None)?;
+                // We don't know the targeted bindings yet, so we use the most lenient
+                PythonInterpreter::find_all(&target, &BridgeModel::Cffi, None)?
+            };
             println!("🐍 {} python interpreter found:", found.len());
             for interpreter in found {
                 println!(" - {}", interpreter);
@@ -372,12 +371,10 @@ fn run() -> Result<()> {
         }
         Opt::Develop {
             bindings,
-            manifest_path,
-            cargo_extra_args,
-            rustc_extra_args,
             release,
             strip,
             extras,
+            cargo_options,
         } => {
             let venv_dir = match (env::var_os("VIRTUAL_ENV"), env::var_os("CONDA_PREFIX")) {
                 (Some(dir), None) => PathBuf::from(dir),
@@ -395,21 +392,15 @@ fn run() -> Result<()> {
                 }
             };
 
-            develop(
-                bindings,
-                &manifest_path,
-                cargo_extra_args,
-                rustc_extra_args,
-                &venv_dir,
-                release,
-                strip,
-                extras,
-            )?;
+            develop(bindings, cargo_options, &venv_dir, release, strip, extras)?;
         }
         Opt::SDist { manifest_path, out } => {
             let build_options = BuildOptions {
-                manifest_path: Some(manifest_path),
                 out,
+                cargo: CargoOptions {
+                    manifest_path: Some(manifest_path),
+                    ..Default::default()
+                },
                 ..Default::default()
             };
             let build_context = build_options.into_build_context(false, false, false)?;
