@@ -17,10 +17,10 @@ mod config;
 /// This snippets will give us information about the python interpreter's
 /// version and abi as json through stdout
 const GET_INTERPRETER_METADATA: &str = include_str!("get_interpreter_metadata.py");
-pub const MINIMUM_PYTHON_MINOR: usize = 6;
+pub const MINIMUM_PYTHON_MINOR: usize = 7;
 /// Be liberal here to include preview versions
 const MAXIMUM_PYTHON_MINOR: usize = 12;
-const MAXIMUM_PYPY_MINOR: usize = 8;
+const MAXIMUM_PYPY_MINOR: usize = 10;
 
 /// Identifies conditions where we do not want to build wheels
 fn windows_interpreter_no_build(
@@ -199,7 +199,7 @@ fn find_all_windows(target: &Target, min_python_minor: usize) -> Result<Vec<Stri
     }
 
     // Fallback to pythonX.Y for Microsoft Store versions
-    for minor in min_python_minor..MAXIMUM_PYTHON_MINOR {
+    for minor in min_python_minor..=MAXIMUM_PYTHON_MINOR {
         if !versions_found.contains(&(3, minor)) {
             let executable = format!("python3.{}.exe", minor);
             if let Some(python_info) = windows_python_info(Path::new(&executable))? {
@@ -278,6 +278,18 @@ pub enum InterpreterKind {
     PyPy,
 }
 
+impl InterpreterKind {
+    /// Is this a CPython interpreter?
+    pub fn is_cpython(&self) -> bool {
+        matches!(self, InterpreterKind::CPython)
+    }
+
+    /// Is this a PyPy interpreter?
+    pub fn is_pypy(&self) -> bool {
+        matches!(self, InterpreterKind::PyPy)
+    }
+}
+
 impl fmt::Display for InterpreterKind {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
@@ -619,6 +631,26 @@ impl PythonInterpreter {
         }
     }
 
+    /// Find all available python interpreters for a given target
+    pub fn find_by_target(
+        target: &Target,
+        min_python_minor: Option<usize>,
+    ) -> Vec<PythonInterpreter> {
+        InterpreterConfig::lookup_target(target)
+            .into_iter()
+            .filter_map(|config| match min_python_minor {
+                Some(min_python_minor) => {
+                    if config.minor < min_python_minor {
+                        None
+                    } else {
+                        Some(Self::from_config(config))
+                    }
+                }
+                None => Some(Self::from_config(config)),
+            })
+            .collect()
+    }
+
     /// Tries to find all installed python versions using the heuristic for the
     /// given platform
     pub fn find_all(
@@ -651,7 +683,7 @@ impl PythonInterpreter {
         let executables = if target.is_windows() {
             find_all_windows(target, min_python_minor)?
         } else {
-            let mut executables: Vec<String> = (min_python_minor..MAXIMUM_PYTHON_MINOR)
+            let mut executables: Vec<String> = (min_python_minor..=MAXIMUM_PYTHON_MINOR)
                 .map(|minor| format!("python3.{}", minor))
                 .collect();
             // Also try to find PyPy for cffi and pyo3 bindings
@@ -660,7 +692,7 @@ impl PythonInterpreter {
                 || bridge.is_bindings("pyo3-ffi")
             {
                 executables.extend(
-                    (min_python_minor..MAXIMUM_PYPY_MINOR).map(|minor| format!("pypy3.{}", minor)),
+                    (min_python_minor..=MAXIMUM_PYPY_MINOR).map(|minor| format!("pypy3.{}", minor)),
                 );
             }
             executables
