@@ -179,7 +179,7 @@ fn add_crate_to_source_distribution(
     {
         bail!(
             "pyproject.toml was not included by `cargo package`. \
-                 Please make sure pyproject.toml is not excluded or build with `--no-sdist`"
+                 Please make sure pyproject.toml is not excluded or build without `--sdist`"
         )
     }
 
@@ -330,7 +330,7 @@ pub fn source_distribution(
 
     writer.add_bytes(
         root_dir.join("PKG-INFO"),
-        metadata21.to_file_contents().as_bytes(),
+        metadata21.to_file_contents()?.as_bytes(),
     )?;
 
     add_data(&mut writer, data)?;
