@@ -24,6 +24,7 @@ pub enum Os {
     Dragonfly,
     Illumos,
     Haiku,
+    Emscripten,
 }
 
 impl fmt::Display for Os {
@@ -38,6 +39,7 @@ impl fmt::Display for Os {
             Os::Dragonfly => write!(f, "DragonFly"),
             Os::Illumos => write!(f, "Illumos"),
             Os::Haiku => write!(f, "Haiku"),
+            Os::Emscripten => write!(f, "Emscripten"),
         }
     }
 }
@@ -57,6 +59,7 @@ pub enum Arch {
     X86,
     X86_64,
     S390X,
+    Wasm32,
 }
 
 impl fmt::Display for Arch {
@@ -70,6 +73,7 @@ impl fmt::Display for Arch {
             Arch::X86 => write!(f, "i686"),
             Arch::X86_64 => write!(f, "x86_64"),
             Arch::S390X => write!(f, "s390x"),
+            Arch::Wasm32 => write!(f, "wasm32"),
         }
     }
 }
@@ -100,6 +104,7 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
         Os::Dragonfly => vec![Arch::X86_64],
         Os::Illumos => vec![Arch::X86_64],
         Os::Haiku => vec![Arch::X86_64],
+        Os::Emscripten => vec![Arch::Wasm32],
     }
 }
 
@@ -123,7 +128,7 @@ impl Target {
     ///
     /// Fails if the target triple isn't supported
     pub fn from_target_triple(target_triple: Option<String>) -> Result<Self> {
-        use target_lexicon::ArmArchitecture;
+        use target_lexicon::{Architecture, ArmArchitecture, OperatingSystem};
 
         let host_triple = get_host_target()?;
         let (platform, triple) = if let Some(ref target_triple) = target_triple {
@@ -139,30 +144,31 @@ impl Target {
         };
 
         let os = match platform.operating_system {
-            target_lexicon::OperatingSystem::Linux => Os::Linux,
-            target_lexicon::OperatingSystem::Windows => Os::Windows,
-            target_lexicon::OperatingSystem::MacOSX { .. }
-            | target_lexicon::OperatingSystem::Darwin => Os::Macos,
-            target_lexicon::OperatingSystem::Netbsd => Os::NetBsd,
-            target_lexicon::OperatingSystem::Freebsd => Os::FreeBsd,
-            target_lexicon::OperatingSystem::Openbsd => Os::OpenBsd,
-            target_lexicon::OperatingSystem::Dragonfly => Os::Dragonfly,
-            target_lexicon::OperatingSystem::Illumos => Os::Illumos,
-            target_lexicon::OperatingSystem::Haiku => Os::Haiku,
+            OperatingSystem::Linux => Os::Linux,
+            OperatingSystem::Windows => Os::Windows,
+            OperatingSystem::MacOSX { .. } | OperatingSystem::Darwin => Os::Macos,
+            OperatingSystem::Netbsd => Os::NetBsd,
+            OperatingSystem::Freebsd => Os::FreeBsd,
+            OperatingSystem::Openbsd => Os::OpenBsd,
+            OperatingSystem::Dragonfly => Os::Dragonfly,
+            OperatingSystem::Illumos => Os::Illumos,
+            OperatingSystem::Haiku => Os::Haiku,
+            OperatingSystem::Emscripten => Os::Emscripten,
             unsupported => bail!("The operating system {:?} is not supported", unsupported),
         };
 
         let arch = match platform.architecture {
-            target_lexicon::Architecture::X86_64 => Arch::X86_64,
-            target_lexicon::Architecture::X86_32(_) => Arch::X86,
-            target_lexicon::Architecture::Arm(arm_arch) => match arm_arch {
+            Architecture::X86_64 => Arch::X86_64,
+            Architecture::X86_32(_) => Arch::X86,
+            Architecture::Arm(arm_arch) => match arm_arch {
                 ArmArchitecture::Arm | ArmArchitecture::Armv6 => Arch::Armv6L,
                 _ => Arch::Armv7L,
             },
-            target_lexicon::Architecture::Aarch64(_) => Arch::Aarch64,
-            target_lexicon::Architecture::Powerpc64 => Arch::Powerpc64,
-            target_lexicon::Architecture::Powerpc64le => Arch::Powerpc64Le,
-            target_lexicon::Architecture::S390x => Arch::S390X,
+            Architecture::Aarch64(_) => Arch::Aarch64,
+            Architecture::Powerpc64 => Arch::Powerpc64,
+            Architecture::Powerpc64le => Arch::Powerpc64Le,
+            Architecture::S390x => Arch::S390X,
+            Architecture::Wasm32 => Arch::Wasm32,
             unsupported => bail!("The architecture {} is not supported", unsupported),
         };
 
@@ -203,8 +209,7 @@ impl Target {
             | (Os::OpenBsd, Arch::X86)
             | (Os::OpenBsd, Arch::X86_64)
             | (Os::OpenBsd, Arch::Aarch64) => {
-                let info = PlatformInfo::new()?;
-                let release = info.release().replace('.', "_").replace('-', "_");
+                let release = self.get_platform_release()?;
                 let arch = match self.arch {
                     Arch::X86_64 => "amd64",
                     Arch::X86 => "i386",
@@ -226,8 +231,7 @@ impl Target {
             (Os::Dragonfly, Arch::X86_64)
             // Haiku
             | (Os::Haiku, Arch::X86_64) => {
-                let info = PlatformInfo::new()?;
-                let release = info.release().replace('.', "_").replace('-', "_");
+                let release = self.get_platform_release()?;
                 format!(
                     "{}_{}_{}",
                     self.os.to_string().to_ascii_lowercase(),
@@ -235,6 +239,7 @@ impl Target {
                     "x86_64"
                 )
             }
+            // Illumos
             (Os::Illumos, Arch::X86_64) => {
                 let info = PlatformInfo::new()?;
                 let mut release = info.release().replace('.', "_").replace('-', "_");
@@ -258,6 +263,7 @@ impl Target {
                     arch
                 )
             }
+            // Linux
             (Os::Linux, _) => {
                 let arch = if self.cross_compiling {
                     self.arch.to_string()
@@ -277,6 +283,7 @@ impl Target {
                 }
                 tags.join(".")
             }
+            // macOS
             (Os::Macos, Arch::X86_64) => {
                 let ((x86_64_major, x86_64_minor), (arm64_major, arm64_minor)) = macosx_deployment_target(env::var("MACOSX_DEPLOYMENT_TARGET").ok().as_deref(), universal2)?;
                 if universal2 {
@@ -305,14 +312,39 @@ impl Target {
                     format!("macosx_{}_{}_arm64", arm64_major, arm64_minor)
                 }
             }
+            // Windows
             (Os::Windows, Arch::X86) => "win32".to_string(),
             (Os::Windows, Arch::X86_64) => "win_amd64".to_string(),
             (Os::Windows, Arch::Aarch64) => "win_arm64".to_string(),
+            // Emscripten
+            (Os::Emscripten, Arch::Wasm32) => {
+                let os_version = env::var("MATURIN_EMSCRIPTEN_VERSION");
+                let release = match os_version {
+                    Ok(os_ver) => os_ver,
+                    Err(_) => emcc_version()?,
+                };
+                let release = release.replace('.', "_").replace('-', "_");
+                format!("emscripten_{}_wasm32", release)
+            }
             (_, _) => panic!("unsupported target should not have reached get_platform_tag()"),
         };
         Ok(tag)
     }
 
+    fn get_platform_release(&self) -> Result<String> {
+        let os = self.os.to_string();
+        let os_version = env::var(format!("MATURIN_{}_VERSION", os.to_ascii_uppercase()));
+        let release = match os_version {
+            Ok(os_ver) => os_ver,
+            Err(_) => {
+                let info = PlatformInfo::new()?;
+                info.release().to_string()
+            }
+        };
+        let release = release.replace('.', "_").replace('-', "_");
+        Ok(release)
+    }
+
     /// Returns the name python uses in `sys.platform` for this architecture.
     pub fn get_python_arch(&self) -> &str {
         match self.arch {
@@ -324,6 +356,7 @@ impl Target {
             Arch::X86 => "i386",
             Arch::X86_64 => "x86_64",
             Arch::S390X => "s390x",
+            Arch::Wasm32 => "wasm32",
         }
     }
 
@@ -339,6 +372,7 @@ impl Target {
             Os::Dragonfly => "dragonfly",
             Os::Illumos => "sunos",
             Os::Haiku => "haiku",
+            Os::Emscripten => "emscripten",
         }
     }
 
@@ -349,7 +383,7 @@ impl Target {
                 PlatformTag::manylinux2014()
             }
             Arch::X86 | Arch::X86_64 => PlatformTag::manylinux2010(),
-            Arch::Armv6L => PlatformTag::Linux,
+            Arch::Armv6L | Arch::Wasm32 => PlatformTag::Linux,
         }
     }
 
@@ -357,7 +391,7 @@ impl Target {
     pub fn pointer_width(&self) -> usize {
         match self.arch {
             Arch::Aarch64 | Arch::Powerpc64 | Arch::Powerpc64Le | Arch::X86_64 | Arch::S390X => 64,
-            Arch::Armv6L | Arch::Armv7L | Arch::X86 => 32,
+            Arch::Armv6L | Arch::Armv7L | Arch::X86 | Arch::Wasm32 => 32,
         }
     }
 
@@ -377,7 +411,8 @@ impl Target {
             | Os::OpenBsd
             | Os::Dragonfly
             | Os::Illumos
-            | Os::Haiku => true,
+            | Os::Haiku
+            | Os::Emscripten => true,
         }
     }
 
@@ -431,6 +466,11 @@ impl Target {
         self.os == Os::Haiku
     }
 
+    /// Returns true if the current platform is Emscripten
+    pub fn is_emscripten(&self) -> bool {
+        self.os == Os::Emscripten
+    }
+
     /// Returns true if the current platform's target env is Musl
     pub fn is_musl_target(&self) -> bool {
         matches!(
@@ -580,6 +620,22 @@ fn macosx_deployment_target(
     Ok((x86_64_ver, arm64_ver))
 }
 
+fn emcc_version() -> Result<String> {
+    use regex::bytes::Regex;
+    use std::process::Command;
+
+    let emcc = Command::new("emcc")
+        .arg("--version")
+        .output()
+        .context("Failed to run emcc to get the version")?;
+    let pattern = Regex::new(r"^emcc .+? (\d+\.\d+\.\d+).*").unwrap();
+    let caps = pattern
+        .captures(&emcc.stdout)
+        .context("Failed to parse emcc version")?;
+    let version = caps.get(1).context("Failed to parse emcc version")?;
+    Ok(String::from_utf8(version.as_bytes().to_vec())?)
+}
+
 #[cfg(test)]
 mod test {
     use super::macosx_deployment_target;
